#include <pstream.h>
#include <iostream>
#include <string>
#include <signal.h>
#include <cstdlib>
#include <sys/wait.h>
#include <setjmp.h>
using namespace std;
using namespace redi;

/*
 * 		calert.cc (Time out program)    		mikeofthenight2003@yahoo.com
 *
 * 		Used in conjuction with a bash script to establish a modem connection
 * 		to a usb port then makes the connection to a (ISP) internet service 
 * 		provider.
 * 		This program establishes the device and provides the timeout.
 * 		 
 * 		Input:	Calert device seconds 
 *
 * 		Where device is the  point to point device	and seconds is the timeout
 * 		time should the connection fail.
 *
 * 		Returns: Status of the connection.
 *
 * 		To build this file first:
 * 		Place the pstream.h file either in the include path to c++ headers
 * 		example: /usr/include/c++/4.4.1/pstream.h. This path will differ for
 * 		different versions of c++.
 * 		Or place it in the current directory where the compiler can find it.
 * 		The latest version of pstreams can be downloaded from sorceforge.net.
 *
 *		Then run:  	g++ -Wall -Os calert.cc -o Calert
 *					strip Calert
 *
 * 		From the Gnome System/Administration/NetworkDeviceMenu/General. 
 * 		Ensure that all users can enable and disable all ethernet and ppp devices
 *
 *		Test the Calert executable from the command line with any device.
 *		Example ./Calert eth1 2 
 *		This should cause eth1 to timeout and display the failed status.
 *		Then again with a longer time period
 *		./Calert eth1 15 
 *		The displayed status should show eth1 up.
 *		Place the Calert executable in the env path where the bash script
 *		can find it.
 *
 */ 		

template class redi::basic_ipstream<char>;
template class redi::basic_pstreambuf<char>;

bool InsertAt(string &str,const string &original,const string &newstr);
static void sig_alrm(int);
static jmp_buf env_alrm;
static int Nm = 0;


inline std::ostream &stream0(std::ostream &stream){
	    stream << "\e[s\e[60D\e[30;1m Ouch...! [" << Nm << "] \e[u\e[0m";
		return stream;
}

inline std::ostream &stream1(std::ostream &stream){
	    stream << "\e[s\e[60D\e[32;1m Ouch...! [" << Nm << "] \e[u\e[0m";
		return stream;
}

void  Si_handler(int sig){

	signal(sig,SIG_IGN);
	Nm++;
	cerr << stream0;
	signal(SIGINT,Si_handler);
}

void  Sq_handler(int sig){

	signal(sig,SIG_IGN);
	Nm++;
	cerr << stream1;
	signal(SIGQUIT,Sq_handler);
}

static void SetupHandler(int sig,void (*handler)(int)){
	struct sigaction action;
	action.sa_handler = handler;
	sigemptyset(&(action.sa_mask));
	sigaddset(&(action.sa_mask),sig);
	action.sa_flags = 0;
	sigaction(sig,&action,0);
}




int main(int argc, char *argv[]) {

SetupHandler(SIGINT, Si_handler);
SetupHandler(SIGQUIT,Sq_handler);

	
int Timeout;   			
string  Device 		= "ifup ";			
string  DeviceDown 	= "ifdown ";			
string  DeviceUp 	= "ifconfig ";			
string  ModemString;
string  str;
unsigned int status=0;

	if (argc ==  3) {
		Device += argv[1];
		DeviceUp += argv[1];
		DeviceDown += argv[1];
		DeviceUp += "|grep RX";
		}

	if (argc ==  3) {
		Timeout = atoi(argv[2]);
	} else {
		cout << "\n\t\tUsage: " << argv[0] << " Device Timeout " <<
				"\n\t\tWhere Device is device to bring up ex: eth0,eth1,ppp0,ppp1..." <<
				"\n\t\tand Timeout is time in seconds before abandoning command\n";
			exit(1);
			}


	if (signal(SIGALRM,sig_alrm) == SIG_ERR){
		cerr << "signal(SIGALRM) error!\n";
		exit(1);
		}

	if (setjmp(env_alrm) != 0) {
		cerr << "\n\e[16C\e[1;37;41mConnection Error:\e[0m Program timed out!\n";
		redi::ipstream ipn(DeviceDown.c_str());  
		ipn.close();
	if (ipn.rdbuf()->exited()){
		status = ipn.rdbuf()->status();
		}
	if (status > 0) return status;
		return 1;
		}

		alarm(Timeout);   /* Start the clock */

				cerr << "\n\e[16CRunning command: " << Device.c_str() << endl;
				pstream ipo(Device.c_str(), pstreambuf::pstderr);
			if (std::getline(ipo, ModemString)){
				InsertAt(ModemString,"n, such","n,\n\e[16Csuch");
				cerr << "\n\e[16C\e[31mError:\e[0m " << ModemString << endl;
				return 1;
				}
				ipo.close();
			if (ipo.rdbuf()->exited()){
				status = ipo.rdbuf()->status();
				}
			if (status > 0) return status;
			
		alarm(0);         /* Stop the clock */

			sleep(2);	  /* allow some time before running connection tests */

			cout << "\n\e[16C\e[30;46mDevice " << argv[1] << " established\e[0m\n";

			sleep(2);	  /* allow some time before running connection tests */

			cout << "\n\t\tRunning command: "<< DeviceUp.c_str() << endl;
			redi::ipstream ipp(DeviceUp.c_str());  
			ModemString.clear();
	while (ipp >> str) {
			ModemString.append(str);
			ModemString.append(" ");
			}
			ipp.close();
		if (ipp.rdbuf()->exited()){
			status = ipp.rdbuf()->status();
			}
		if (status > 0) return status;

			InsertAt(ModemString,"RX bytes","\n\e[16CRX bytes");
			cout << "\n\t\t" << argv[1] << " reports the following:\n\n\e[16C" 
 				 << ModemString.c_str() << endl;

return 0;
}

static void sig_alrm(int signo){
	longjmp(env_alrm,1);
}

bool InsertAt(string &str,const string &original,const string &newstr){
		string::size_type indx;
		indx = str.find(original);
	if (indx != string::npos){
		str.replace(indx,original.size(),newstr);
	return true;
	}
	return false;
}

